/*
 * This file is generated by objective.metadata
 *
 * Last update: Fri May 15 18:01:06 2015
 */

static void __attribute__((__used__)) use_protocols(void)
{
    PyObject* p __attribute__((__unused__));
#if PyObjC_BUILD_RELEASE >= 1010
    p = PyObjC_IdToPython(@protocol(NSAccessibility));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSAccessibilityButton));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSAccessibilityCheckBox));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSAccessibilityContainsTransientUI));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSAccessibilityElement));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSAccessibilityGroup));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSAccessibilityImage));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSAccessibilityLayoutArea));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSAccessibilityLayoutItem));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSAccessibilityList));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSAccessibilityNavigableStaticText));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSAccessibilityOutline));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSAccessibilityProgressIndicator));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSAccessibilityRadioButton));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSAccessibilityRow));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSAccessibilitySlider));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSAccessibilityStaticText));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSAccessibilityStepper));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSAccessibilitySwitch));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSAccessibilityTable));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSAlertDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSAnimatablePropertyContainer));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSAnimationDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSAppearanceCustomization));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSApplicationDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSBrowserDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSChangeSpelling));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSCollectionViewDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSColorPickingCustom));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSColorPickingDefault));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSComboBoxCellDataSource));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSComboBoxDataSource));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSComboBoxDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSControlTextEditingDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSDatePickerCellDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSDockTilePlugIn));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSDraggingDestination));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSDraggingInfo));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSDraggingSource));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSDrawerDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSGestureRecognizerDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSGlyphStorage));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSIgnoreMisspelledWords));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSImageDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSInputServerMouseTracker));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSInputServiceProvider));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSLayoutManagerDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSMatrixDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSMenuDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSOpenSavePanelDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSOutlineViewDataSource));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSOutlineViewDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSPageControllerDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSPasteboardItemDataProvider));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSPasteboardReading));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSPasteboardWriting));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSPathCellDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSPathControlDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSPopoverDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSPrintPanelAccessorizing));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSRuleEditorDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSSeguePerforming));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSServicesMenuRequestor));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSSharingServiceDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSSharingServicePickerDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSSoundDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSSpeechRecognizerDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSSpeechSynthesizerDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSSplitViewDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSStackViewDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSTabViewDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSTableViewDataSource));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSTableViewDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSTextAttachmentCell));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSTextDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSTextFieldDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSTextFinderBarContainer));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSTextFinderClient));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSTextInput));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSTextInputClient));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSTextLayoutOrientationProvider));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSTextStorageDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSTextViewDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSTokenFieldCellDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSTokenFieldDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSToolbarDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSUserInterfaceItemIdentification));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSUserInterfaceItemSearching));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSUserInterfaceValidations));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSValidatedUserInterfaceItem));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSViewControllerPresentationAnimator));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSWindowDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSWindowRestoration));
    Py_XDECREF(p);
#endif /* PyObjC_BUILD_RELEASE >= 1010 */
#if PyObjC_BUILD_RELEASE >= 1011
    p = PyObjC_IdToPython(@protocol(NSAlignmentFeedbackToken));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSCollectionViewElement));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSCollectionViewDelegateFlowLayout));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSSearchFieldDelegate));
    Py_XDECREF(p);
#endif
#if PyObjC_BUILD_RELEASE >= 1012
    p = PyObjC_IdToPython(@protocol(NSCollectionViewSectionHeaderView));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSFilePromiseProviderDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSCloudSharingServiceDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSCloudSharingValidation));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSCandidateListTouchBarItemDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSScrubberDataSource));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSScrubberDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSScrubberFlowLayoutDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSSharingServicePickerTouchBarItemDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSTouchBarDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSTouchBarProvider));
    Py_XDECREF(p);
#endif
#if PyObjC_BUILD_RELEASE >= 1011
    p = PyObjC_IdToPython(@protocol(NSTextAttachmentContainer));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSCollectionViewDataSource));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSSpringLoadingDestination));
    Py_XDECREF(p);
#endif
#if PyObjC_BUILD_RELEASE >= 1013
    p = PyObjC_IdToPython(@protocol(NSAccessibilityElementLoading));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSAccessibilityCustomRotorItemSearchDelegate));
    Py_XDECREF(p);
    // p = PyObjC_IdToPython(@protocol(NSAccessibilityCustomRotorItemLoadDelegate));
    // Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSCollectionViewPrefetching));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSUserInterfaceCompression));
    Py_XDECREF(p);
#endif
#if PyObjC_BUILD_RELEASE >= 1014
    p = PyObjC_IdToPython(@protocol(NSFontChanging));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSEditor));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSEditorRegistration));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSPasteboardTypeOwner));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSStandardKeyBindingResponding));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSToolbarItemValidation));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSViewLayerContentScaleDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSViewToolTipOwner));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSUserActivityRestoring));
    Py_XDECREF(p);
#endif
#if PyObjC_BUILD_RELEASE >= 1014
#endif
#if PyObjC_BUILD_RELEASE >= 1015
    p = PyObjC_IdToPython(@protocol(NSCollectionLayoutContainer));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSTextInputTraits));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSSharingServicePickerToolbarItemDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSCollectionLayoutEnvironment));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSCollectionLayoutVisibleItem));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSTextCheckingClient));
    Py_XDECREF(p);
#endif
#if PyObjC_BUILD_RELEASE >= 1200
    p = PyObjC_IdToPython(@protocol(NSTextElementProvider));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSTextContentManagerDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSTextContentStorageDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSTextLayoutManagerDelegate));
    Py_XDECREF(p);
#endif
}
