/*
 * This file is generated by objective.metadata
 *
 * Last update: Fri May 15 18:01:12 2015
 */

static void __attribute__((__used__)) use_protocols(void)
{
    PyObject* p __attribute__((__unused__));
#if PyObjC_BUILD_RELEASE >= 1005
    p = PyObjC_IdToPython(@protocol(NSObject));
    Py_XDECREF(p);
#endif /* PyObjC_BUILD_RELEASE >= 1005 */
#if PyObjC_BUILD_RELEASE >= 1010
    p = PyObjC_IdToPython(@protocol(NSCacheDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSCoding));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSConnectionDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSCopying));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSDecimalNumberBehaviors));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSDiscardableContent));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSFastEnumeration));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSFileManagerDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSFilePresenter));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSKeyedArchiverDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSKeyedUnarchiverDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSLocking));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSMachPortDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSMetadataQueryDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSMutableCopying));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSNetServiceBrowserDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSNetServiceDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSPortDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSSecureCoding));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSSpellServerDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSStreamDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSURLAuthenticationChallengeSender));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSURLConnectionDataDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSURLConnectionDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSURLConnectionDownloadDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSURLDownloadDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSURLHandleClient));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSURLProtocolClient));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSURLSessionDataDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSURLSessionDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSURLSessionDownloadDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSURLSessionTaskDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSUserNotificationCenterDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSXMLParserDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSXPCListenerDelegate));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSXPCProxyCreating));
    Py_XDECREF(p);
#endif /* PyObjC_BUILD_RELEASE >= 1010 */
#if PyObjC_BUILD_RELEASE >= 1011
    p = PyObjC_IdToPython(@protocol(NSProgressReporting));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSURLSessionStreamDelegate));
    Py_XDECREF(p);
#endif /* PyObjC_BUILD_RELEASE >= 1011 */
#if PyObjC_BUILD_RELEASE >= 1010
    p = PyObjC_IdToPython(@protocol(NSExtensionRequestHandling));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSUserActivityDelegate));
    Py_XDECREF(p);
#endif /* PyObjC_BUILD_RELEASE >= 1010 */
#if PyObjC_BUILD_RELEASE >= 1013
    p = PyObjC_IdToPython(@protocol(NSItemProviderWriting));
    Py_XDECREF(p);
    p = PyObjC_IdToPython(@protocol(NSItemProviderReading));
    Py_XDECREF(p);
#endif
#if PyObjC_BUILD_RELEASE >= 1015
    p = PyObjC_IdToPython(@protocol(NSURLSessionWebSocketDelegate));
    Py_XDECREF(p);
#endif
}
