import AppDelegate  # noqa: F401
import MyBaseGradientView  # noqa: F401
import MyBezierGradientView  # noqa: F401
import MyRectGradientView  # noqa: F401
import MyWindowController  # noqa: F401
from PyObjCTools import AppHelper

AppHelper.runEventLoop()
